'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var core = require('@angular/core');

function initProps(target) {
  target.schema.attrs.forEach((key) => {
    Object.defineProperty(target, key, {
      get() {
        return target.component[key];
      },
      set(value) {
        target.component[key] = value;
      }
    });
  });
}
function initEvents(target) {
  const schema = target.schema;
  schema.events.forEach((outputEvent) => {
    target.component[schema.outputs[outputEvent]].subscribe((info) => {
      target.dispatchEvent(new CustomEvent(outputEvent, {detail: info}));
    });
  });
}

function createStyle(styles) {
  const style = new CSSStyleSheet();
  style.replaceSync(styles.join("\n"));
  return style;
}
function createSchema(component, options2) {
  const {inputs, styles, outputs} = component["ɵcmp"];
  const directives = (options2 == null ? void 0 : options2.directives) ? options2.directives.filter((t) => t["ɵdir"]) : [];
  const components = (options2 == null ? void 0 : options2.directives) ? options2.directives.filter((t) => t["ɵcmp"]) : [];
  const pipes = (options2 == null ? void 0 : options2.pipes) ? options2.pipes : [];
  const attrs = Object.keys(inputs);
  const events = Object.keys(outputs);
  const style = createStyle(styles);
  const props = new Map();
  attrs.forEach((attr) => props.set(attr, component[attr]));
  return {attrs, events, props, style, directives, components, pipes, outputs};
}
const schema = new Map();

function autoChangeDetection(component, props, attrs) {
  attrs.forEach((key) => {
    props.set(key, component[key]);
    Object.defineProperty(component, key, {
      get() {
        return props.get(key);
      },
      set(value) {
        props.set(key, value);
        core.ɵdetectChanges(component);
      }
    });
  });
  return component;
}

function renderNgComponent(componentType, options2) {
  const type = componentType["ɵcmp"];
  const {directives, pipes, props, attrs} = schema.get(type.id);
  type.directiveDefs = directives.map((d) => d["ɵdir"]);
  type.pipeDefs = pipes.map((p) => p["ɵpipe"]);
  const component = core.ɵrenderComponent(componentType, {
    ...options2 != null ? options2 : {},
    rendererFactory: null
  });
  return autoChangeDetection(component, props, attrs);
}

class NgxElement extends HTMLElement {
  constructor(type, options2) {
    super();
    const {id, encapsulation} = type["ɵcmp"];
    this.schema = schema.get(id);
    const host = encapsulation === core.ViewEncapsulation.ShadowDom ? this.attachShadow({mode: "open"}) : this;
    host.adoptedStyleSheets = [schema.get(id).style];
    this.component = renderNgComponent(type, {...options2 != null ? options2 : {}, host});
  }
  attributeChangedCallback(name, oldValue, newValue) {
    if (this.schema.attrs.includes(name)) {
      this[name] = newValue;
    }
  }
}

function createCustomElement(componentType, options2) {
  const type = componentType["ɵcmp"];
  const schemaResult = createSchema(componentType, options2);
  const {components, directives, attrs} = schemaResult;
  (options2 == null ? void 0 : options2.directives) && components.forEach((type2) => renderCustomElement(type2, {...options2, directives}));
  schema.set(type.id, schemaResult);
  return class CustomElement extends NgxElement {
    constructor() {
      super(componentType, options2);
      initProps(this);
      initEvents(this);
    }
    static get observedAttributes() {
      return [...attrs];
    }
  };
}
function renderCustomElement(componentType, options2) {
  customElements.define(componentType["ɵcmp"].selectors[0][0], createCustomElement(componentType, options2));
}

exports.NgxElement = NgxElement;
exports.createCustomElement = createCustomElement;
exports.renderCustomElement = renderCustomElement;
